// Copyright(c) 2005 Analog Devices, Inc. All Rights Reserved.
// This software is proprietary and confidential to Analog Devices, Inc. and its licensors.

// File    : $Id: //depot/development/visualaudio/Platforms/EZKit_21369_Basic/2.5.0/Drivers/Audio/AD1835.h#2 $ 
// Part of : VisualAudio V2.5.0 
// Updated : $Date: 2006/10/12 $ by $Author: Fernando $

#ifndef _AD1835_H_
#define _AD1835_H_

/******************************************************************************************************************
    
    
     Owner:            Analog Devices Inc

    File Group:        Visual Audio Platform Library
    File name:        AD1835.h
      Description:    AD1835 CODEC specification

    Revision:

    E. Yin            29 Aug 2005        Initial Revision
******************************************************************************************************************/



/*
 * AD1835 codec SPI control/status register definitions. 
 */
/** AD1835 register mask - Read register flag. */
#define READ_REG                        0x0800
/** AD1835 register mask - Write register flag. */
#define WRITE_REG                        0x0000
/** AD1835 register mask - DAC Control 1 register. */
#define DAC_CONTROL1                    0x0000
/** AD1835 register mask - DAC Control 2 register. */
#define DAC_CONTROL2                    0x1000
/** AD1835 register mask - DAC volume - left 1 register. */
#define DAC_VOLUME_L1                    0x2000
/** AD1835 register mask - DAC volume - right 1 register. */
#define DAC_VOLUME_R1                    0x3000
/** AD1835 register mask - DAC volume - left 2 register. */
#define DAC_VOLUME_L2                    0x4000
/** AD1835 register mask - DAC volume - right 2 register. */
#define DAC_VOLUME_R2                    0x5000
/** AD1835 register mask - DAC volume - left 3 register. */
#define DAC_VOLUME_L3                    0x6000
/** AD1835 register mask - DAC volume - right 3 register. */
#define DAC_VOLUME_R3                    0x7000
/** AD1835 register mask - DAC volume - left 4 register. */
#define DAC_VOLUME_L4                    0x8000
/** AD1835 register mask - DAC volume - right 4 register. */
#define DAC_VOLUME_R4                    0x9000
/** AD1835 register mask - ADC peak level left register. */
#define ADC_PEAK_LEVEL_LEFT                0xA000
/** AD1835 register mask - ADC peak level right register. */
#define ADC_PEAK_LEVEL_RIGHT            0xB000
/** AD1835 register mask - ADC Control 1 register. */
#define ADC_CONTROL1                    0xC000
/** AD1835 register mask - ADC Control 2 register. */
#define ADC_CONTROL2                    0xD000
/** AD1835 register mask - ADC Control 3 register. */
#define ADC_CONTROL3                    0xE000


//DAC Control 1 Flag
/** AD1835 DAC Control 1 flag - Power down */
#define DAC_CONTROL1_POWERDOWN            0x0004
/** AD1835 DAC Control 1 flag - Data word width (24 bits) */
#define DAC_CONTROL1_DATAWIDTH_24        0x0000
/** AD1835 DAC Control 1 flag - Data word width (20 bits) */
#define DAC_CONTROL1_DATAWIDTH_20        0x0008
/** AD1835 DAC Control 1 flag - Data word width (16 bits) */
#define DAC_CONTROL1_DATAWIDTH_16        0x0010
/** AD1835 DAC Control 1 flag - Serial Mode (I2S) */
#define DAC_CONTROL1_SERMODE_I2S        0x0000
/** AD1835 DAC Control 1 flag - Serial Mode (RJ) */
#define DAC_CONTROL1_SERMODE_RJ            0x0020
/** AD1835 DAC Control 1 flag - Serial Mode (DSP) */
#define DAC_CONTROL1_SERMODE_DSP        0x0040
/** AD1835 DAC Control 1 flag - Serial Mode (LJ) */
#define DAC_CONTROL1_SERMODE_LJ            0x0060
/** AD1835 DAC Control 1 flag - Serial Mode (Pack Mode 256) */
#define DAC_CONTROL1_SERMODE_PACK256    0x0080
/** AD1835 DAC Control 1 flag - Serial Mode (Pack Mode 128) */
#define DAC_CONTROL1_SERMODE_PACK128    0x00A0
/** AD1835 DAC Control 1 flag - De-Emphasis (None) */
#define DAC_CONTROL1_DEEMP_NONE            0x0000
/** AD1835 DAC Control 1 flag - De-Emphasis (44.1 Khz) */
#define DAC_CONTROL1_DEEMP_44_1Khz        0x0100
/** AD1835 DAC Control 1 flag - De-Emphasis (32 Khz) */
#define DAC_CONTROL1_DEEMP_32Khz        0x0200
/** AD1835 DAC Control 1 flag - De-Emphasis (48 Khz) */
#define DAC_CONTROL1_DEEMP_48Khz        0x0300

/** AD1835 ADC Control 1 flag - Sample Rate (48 Khz). */
#define DAC_CONTROL1_SAMPRATE_48Khz        0x0000
/** AD1835 ADC Control 1 flag - Sample Rate (96 Khz). */
#define DAC_CONTROL1_SAMPRATE_96Khz        0x0001
/** AD1835 ADC Control 1 flag - Sample Rate (192 Khz). */
#define DAC_CONTROL1_SAMPRATE_192Khz    0x0002



// DAC Control 2 Flag
// Replicate output of DAC 1/2 on 3/4, 5/6 & 7/8
#define DAC_CONTROL2_REPLICATE            0x0100 
/** AD1835 DAC Control 2 flag - DAC - left 1 mute. */
#define DAC_CONTROL2_MUTEDAC_L1            0x0001
/** AD1835 DAC Control 2 flag - DAC - right 1 mute. */
#define DAC_CONTROL2_MUTEDAC_R1            0x0002
/** AD1835 DAC Control 2 flag - DAC - left 2 mute. */
#define DAC_CONTROL2_MUTEDAC_L2            0x0004
/** AD1835 DAC Control 2 flag - DAC - right 2 mute. */
#define DAC_CONTROL2_MUTEDAC_R2            0x0008
/** AD1835 DAC Control 2 flag - DAC - left 3 mute. */
#define DAC_CONTROL2_MUTEDAC_L3            0x0010
/** AD1835 DAC Control 2 flag - DAC - right 3 mute. */
#define DAC_CONTROL2_MUTEDAC_R3            0x0020
/** AD1835 DAC Control 2 flag - DAC - left 4 mute. */
#define DAC_CONTROL2_MUTEDAC_L4            0x0040
/** AD1835 DAC Control 2 flag - DAC - right 4 mute. */
#define DAC_CONTROL2_MUTEDAC_R4            0x0080


//DAC Volume Control - 10-bit granularity (1024 levels)
#define DAC_VOLUME_MIN      (0x000)
#define DAC_VOLUME_LOW      (0X100)
#define DAC_VOLUME_MED      (0X200)
#define DAC_VOLUME_HI       (0X300)
#define DAC_VOLUME_MAX      (0x3FF)
#define DAC_VOLUME_MASK     (0x3FF)  // Volume in dB is in 10 LSBs
                                     //   3FF = 0 dBFS = 1023/1023
                                     //   3FE = -0.01 dBFS = 1022/1023
                                     //      ...
                                     //   002 = -50.7 dBFS = 3/1023
                                     //   001 = -54.2 dBFS = 2/1023

/** AD1835 ADC Control 1 flag - Sample Rate (48 Khz). */
#define ADC_CONTROL1_SAMPRATE_48Khz        0x0000
/** AD1835 ADC Control 1 flag - Sample Rate (96 Khz). */
#define ADC_CONTROL1_SAMPRATE_96Khz        0x0040
/** AD1835 ADC Control 1 flag - Power down. */
#define ADC_CONTROL1_POWERDOWN            0x0080
/** AD1835 ADC Control 1 flag - Filter (High pass). */
#define ADC_CONTROL1_FILTER_HIGHPASS    0x0100


#define ADCMUTER   (0x0002)  // Mute right channel from ADC
#define ADCMUTEL   (0x0001)  // Mute left channel from ADC

/** AD1835 ADC Control 2 flag - Data word width (24 bits). */
#define ADC_CONTROL2_WORDWIDTH_24        0x0000
/** AD1835 ADC Control 2 flag - Data word width (20 bits). */
#define ADC_CONTROL2_WORDWIDTH_20        0x0010
/** AD1835 ADC Control 2 flag - Data word width (16 bits). */
#define ADC_CONTROL2_WORDWIDTH_16        0x0020
/** AD1835 ADC Control 2 flag - Serial Mode (I2S) */
#define ADC_CONTROL2_SERMODE_I2S        0x0000
/** AD1835 ADC Control 2 flag - Serial Mode (RJ) */
#define ADC_CONTROL2_SERMODE_RJ            0x0040
/** AD1835 ADC Control 2 flag - Serial Mode (DSP) */
#define ADC_CONTROL2_SERMODE_DSP        0x0080
/** AD1835 ADC Control 2 flag - Serial Mode (LJ) */
#define ADC_CONTROL2_SERMODE_LJ            0x00C0
/** AD1835 ADC Control 2 flag - Serial Mode (Packed 256) */
#define ADC_CONTROL2_SERMODE_PACK256    0x0100
/** AD1835 ADC Control 2 flag - Serial Mode (Packed 128) */
#define ADC_CONTROL2_SERMODE_PACK128    0x0180
/** AD1835 ADC Control 2 flag - Master/Slave AUX Mode (Slave) */
#define ADC_CONTROL2_SLAVE                0x0000
/** AD1835 ADC Control 2 flag - Master/Slave AUX Mode (Master) */
#define ADC_CONTROL2_MASTER                0x0200


/** AD1835 ADC Control 3 flag - Enable reads of peak ADC levels */
#define ADC_CONTROL3_PEAK_READ_ENABLE    0x0020
/** AD1835 ADC Control 3 flag - Clock Mode (Internal MCLK = external MCLK x 2) */
#define ADC_CONTROL3_CLK_IMCLKx2        0x0000
/** AD1835 ADC Control 3 flag - Clock Mode (Internal MCLK = external MCLK) */
#define ADC_CONTROL3_CLK_IMCLKx1        0x0080
/** AD1835 ADC Control 3 flag - Clock Mode (Internal MCLK = external MCLK x 2/3) */
#define ADC_CONTROL3_CLK_IMCLKx23        0x0100

#endif //_AD1835_H_
