// Copyright(c) 2005 Analog Devices, Inc. All Rights Reserved.
// This software is proprietary and confidential to Analog Devices, Inc. and its licensors.

// File    : $Id: //depot/development/visualaudio/Platforms/EZKit_21369_Basic/2.5.0/Drivers/Audio/AD1835_SPI.c#3 $ 
// Part of : VisualAudio V2.5.0 
// Updated : $Date: 2006/10/12 $ by $Author: Fernando $

/******************************************************************************************************************
    
    
     Owner:            Analog Devices Inc

    File Group:        Visual Audio Platform Library
    File name:        AD1835_SPI.C
      Description:    AD1835 CODEC configuration via SPI

    Revision:

    E. Yin            29 Aug 2005        Initial Revision
******************************************************************************************************************/
#include "Driver.h"
#include "ad1835.h"
#include "SPIDriver.h"
#include "VALayouts.h"
#include "Basic.h"

#ifndef Layout1_INITIAL_ASSUMED_SAMPLE_RATE
#define ADC_SAMPLING_RATE 48000
#else 
#define ADC_SAMPLING_RATE Layout1_INITIAL_ASSUMED_SAMPLE_RATE
#endif

#if (ADC_SAMPLING_RATE == 96000)

#define ANALOG_DAC_CONTROL1_SAMPRATE   \
            (DAC_CONTROL1_SAMPRATE_96Khz)

#define DIGITAL_DAC_CONTROL1_SAMPRATE   \
            (DAC_CONTROL1_SAMPRATE_48Khz |      \
            DAC_CONTROL1_SAMPRATE_96Khz |       \
            DAC_CONTROL1_SAMPRATE_192Khz)
            
#define ADC_CONTROL1_SAMPRATE   ADC_CONTROL1_SAMPRATE_96Khz

#define ADC_CONTROL3_CLK        ADC_CONTROL3_CLK_IMCLKx1

#else

#define ANALOG_DAC_CONTROL1_SAMPRATE   \
            (DAC_CONTROL1_SAMPRATE_48Khz |      \
            DAC_CONTROL1_SAMPRATE_96Khz |       \
            DAC_CONTROL1_SAMPRATE_192Khz)
            
#define DIGITAL_DAC_CONTROL1_SAMPRATE   ANALOG_DAC_CONTROL1_SAMPRATE

#define ADC_CONTROL1_SAMPRATE   ADC_CONTROL1_SAMPRATE_48Khz

#define ADC_CONTROL3_CLK        ADC_CONTROL3_CLK_IMCLKx2

#endif

///////////////////////////////////////////////////////////////////
// AD1835A CODEC Initialization Routine
///////////////////////////////////////////////////////////////////
/** Length of the init commands buffer. */
#define INIT_CMDS_LEN    14
const unsigned short g_SPDIF_AD1835_TX [INIT_CMDS_LEN] = 
{
            //Configure DAC control 1 Register
            DAC_CONTROL1|WRITE_REG|DAC_CONTROL1_POWERDOWN,
            DAC_CONTROL1|WRITE_REG                    |
            DAC_CONTROL1_SERMODE_I2S                 |
            DAC_CONTROL1_DATAWIDTH_24                |
            DIGITAL_DAC_CONTROL1_SAMPRATE,
            
           // Write the DAC control register 2 to clear any mutes.
            DAC_CONTROL2|WRITE_REG,
            
            // Set the dac volumes to MAX
            DAC_VOLUME_L1|WRITE_REG    |DAC_VOLUME_MAX,
            DAC_VOLUME_R1|WRITE_REG    |DAC_VOLUME_MAX,
            DAC_VOLUME_L2|WRITE_REG    |DAC_VOLUME_MAX,                                    
            DAC_VOLUME_R2|WRITE_REG    |DAC_VOLUME_MAX,
            DAC_VOLUME_L3|WRITE_REG    |DAC_VOLUME_MAX,
            DAC_VOLUME_R3|WRITE_REG    |DAC_VOLUME_MAX,
            DAC_VOLUME_L4|WRITE_REG    |DAC_VOLUME_MAX,
            DAC_VOLUME_R4|WRITE_REG    |DAC_VOLUME_MAX,                                                                                                                                                                                                                                                
            
            //Configure ADC control 1 register
            ADC_CONTROL1| WRITE_REG |ADC_CONTROL1_SAMPRATE,

            //Configure ADC control 2 register
            ADC_CONTROL2| WRITE_REG                 |
            ADC_CONTROL2_SERMODE_I2S                |
            ADC_CONTROL2_WORDWIDTH_24,

            //Configure ADC control 3 register
            ADC_CONTROL3| WRITE_REG                 |
            ADC_CONTROL3_CLK,
};

const unsigned short g_Analog_AD1835_TX [INIT_CMDS_LEN] = 
{
            //Configure DAC control 1 Register
            DAC_CONTROL1|WRITE_REG|DAC_CONTROL1_POWERDOWN,
            DAC_CONTROL1|WRITE_REG                    |
            DAC_CONTROL1_SERMODE_I2S                 |
            DAC_CONTROL1_DATAWIDTH_24                |
            ANALOG_DAC_CONTROL1_SAMPRATE,
            
            // Write the DAC control register 2 to clear any mutes.
            DAC_CONTROL2|WRITE_REG,
            
            // Set the dac volumes to MAX
            DAC_VOLUME_L1|WRITE_REG    |DAC_VOLUME_MAX,
            DAC_VOLUME_R1|WRITE_REG    |DAC_VOLUME_MAX,
            DAC_VOLUME_L2|WRITE_REG    |DAC_VOLUME_MAX,                                    
            DAC_VOLUME_R2|WRITE_REG    |DAC_VOLUME_MAX,
            DAC_VOLUME_L3|WRITE_REG    |DAC_VOLUME_MAX,
            DAC_VOLUME_R3|WRITE_REG    |DAC_VOLUME_MAX,
            DAC_VOLUME_L4|WRITE_REG    |DAC_VOLUME_MAX,
            DAC_VOLUME_R4|WRITE_REG    |DAC_VOLUME_MAX,                                                                                                                                                                                                                                                
            
            //Configure ADC control 1 register
            ADC_CONTROL1| WRITE_REG |ADC_CONTROL1_SAMPRATE,

            //Configure ADC control 2 register
            ADC_CONTROL2| WRITE_REG                 |
            ADC_CONTROL2_SERMODE_I2S                |
            ADC_CONTROL2_WORDWIDTH_24,

            //Configure ADC control 3 register
            ADC_CONTROL3| WRITE_REG                 |
            ADC_CONTROL3_CLK,
};

#ifndef NO_OPTIMIZE
#pragma optimize_for_space
#endif
void Init_AD1835_via_SPI(AudioIOMode iMode)
{
    SPIConfigParam spi_config_param;
    spi_config_param.unBaudrate = 100;
    spi_config_param.unSPIFlag = 63240; //0xF708       //256;  //0x100
    spi_config_param.unSPIControl = (SPIEN | SPIMS | MSBF | WL16 | TIMOD1);

    /* Now setup the SPI Control register : 0x5281*/
    //The default setting of CPHASE is 0.
    //if CPHASE set(=1), the slave-select flags manually controlled in software
    //by setting and clearing the appropriate SPIFLGx bit in SPIFLG regiter
    //if CPHASE set(=0), the slave-select flags are enabled by DSxEN bits of SPIFLG
    //are automatically asserted by the SPI port when appropriate, and the SPIFLGx
    //bit is ignored.
   // WRPORT(SPICTL) = (SPIEN | SPIMS | MSBF | WL16 | TIMOD1) ;
    
    //initialize the SPI port
    SPI_Init(&spi_config_param);

    if (iMode == AudioIOMode_SPDIF)
        SPI_Write_16Bit_Words(g_SPDIF_AD1835_TX, INIT_CMDS_LEN);
    else
        SPI_Write_16Bit_Words(g_Analog_AD1835_TX, INIT_CMDS_LEN);

    //disable the SPI port when done
    SPI_Clear();
    
}
