// Copyright(c) 2005 Analog Devices, Inc. All Rights Reserved.
// This software is proprietary and confidential to Analog Devices, Inc. and its licensors.

// File    : $Id: //depot/development/visualaudio/Platforms/EZKit_21369_Basic/2.5.0/Drivers/Audio/AudioDriver.c#5 $ 
// Part of : VisualAudio V2.5.0 
// Updated : $Date: 2006/10/12 $ by $Author: Fernando $

/******************************************************************************************************************
    
    
     Owner:            Analog Devices Inc

    File Group:        Visual Audio Platform Library
    File name:        AudioDriver.c
      Description:    Audio Driver implementation for Basic version of AVR framework
    Platform:        ADSP-21369 EZKit
    I/O Support:    SPDIF In, 2 Channel analog In, SPDIF out and 8 Channel Analog out    

******************************************************************************************************************
    Revision:

    E. Yin            30 Aug 2005        Initial Revision
    E. Yin            03 Oct 2005        Modified for simplified version of framework
    E. Yin            28 Oct 2005        Renamed xxx_IRQ to xxx_ISR for interrupt handler
******************************************************************************************************************/

#include "Driver.h" //common header file for drivers
#include "Basic.h"            //for Basic framework
#include "AudioDriverPrivate.h"
#include <builtins.h>
#include <string.h>
#include <stdlib.h>
#include <signal.h>
asm("#include \"processor.h\"");

#include <sysreg.h>
#include "VALayouts.h"

#define HANDLE_RECEIVE_ERROR

static AudioIOMode g_iAudioInputMode;        //current audio input mode

#if defined(DATA_FORMAT_LJ)
#define SPORT_DATA_FORMAT LAFS    //left justifyied - Active High frame syncs and Late Frame Sync
#elif defined(DATA_FORMAT_I2S)
#define SPORT_DATA_FORMAT L_FIRST   //I2S format(set FRFS bit - Active low frame syncs)
#else
#error "No serial data format defined in AudioDriverPrivate.h!"
#endif

void SYSAudio_Clear(void);

//The number of available input channels for analog input
#define ANALOG_IN_CHANNEL_COUNT    2

//The number of channels in SPDIF input
#define SPDIF_IN_CHANNEL_COUNT 2  //Digital data

/*** constant value for output ***/
// Number of transmiting stereo pairs
#define ANALOG_OUT_STEREO_PAIR_COUNT 4

/**
 * DMA Transfer Control Block.
 *
 * This structure defines the memory layout of the DMA Transfer Control Block
 * (TCB). The TCB is used when a DMA operation completes to automatically 
 * re-load the DMA registers and start another DMA operation.
 */
typedef struct
{
    /** Chain pointer for next TCB. */
    unsigned int m_CPSPx;
    /** Length of source buffer. */
    unsigned int m_CSPx;
    /** Source buffer step size. */
    unsigned int m_IMSPx;
    /** Start address for data buffer. */
    unsigned int m_IISPx;
} DMA_TCB;
#define CPX_ADDR_MASK (0x7FFFF)
#define CPX_IRQ_FLAG  (0x80000)
#define CPX_OFFSET 3

/** CPx Register mask for the TCB address. */
#define CPx_ADDR_MASK            CPX_ADDR_MASK
/** CPx Register bit for the IRQ flag. */
#define CPx_IRQ_FLAG            CPX_IRQ_FLAG

unsigned int DMA_MakeCPx(const DMA_TCB *pTCB, int bIRQ)
{
    // Calculate the base address.
    unsigned int CPx = ((unsigned int)(pTCB) + CPX_OFFSET) & CPx_ADDR_MASK;

    // Set the IRQ flag bit.
    if (bIRQ != 0) CPx |= CPx_IRQ_FLAG;

    // Return the register.
    return CPx;
}

/*
 * The following section describes the Transfer Control Blocks used by the DMA
 * controller to DMA data to and from the codec. Transfer Control Blocks (TCBs)
 * are read by the DMA controller to initialize the DMA registers for the next
 * transfer operation. The following TCBs are configured to act as
 * circular-buffered DMA on the transmit and receive buffers.
 */

/** size of the transmit buffer. There are two of these (double-buffering) */
#define RX_BUF_SIZE     (Layout1_TICK_SIZE * SPDIF_IN_CHANNEL_COUNT)

//////////////////////////////////////////////////////////////////////////
// Allocating Memory for Receiving buffer
//
static SEG_DRV_DMA_DATA_1 int g_SPORT0A_RX_1[RX_BUF_SIZE];
static SEG_DRV_DMA_DATA_2 int g_SPORT0A_RX_2[RX_BUF_SIZE];

#define SPORT0A_RX_1_START_ADDRESS    (((int)&g_SPORT0A_RX_1[0]) & 0x7FFFF)
#define SPORT0A_RX_1_END_ADDRESS    (((int)&g_SPORT0A_RX_1[RX_BUF_SIZE-1]) & 0x7FFFF)


////////////////////////////////////////////////////////////////////////
// 
//       Setting up DMA Control Block for 
////////////////////////////////////////////////////////////////////////
// SPORT0 channel pair A receives TCB (Buffer A).
////////////////////////////////////////////////////////////////////////
static DMA_TCB g_sport0_rx_tcb_A =
{
    0,                                                    // m_CPSPx
    RX_BUF_SIZE,                                    // m_CSPx
    1,                                                     // m_IMSPx
    (unsigned int)(&g_SPORT0A_RX_1[0])                    // m_IISPx
};

// SPORT 0 receive TCB (Buffer B). 
static DMA_TCB g_sport0_rx_tcb_A_1 =
{
    0,                                                        // m_CPSPx
    RX_BUF_SIZE,                                        // m_CSPx
    1,                                                         // m_IMSPx
    (unsigned int)(&g_SPORT0A_RX_2[0])                        // m_IISPx
};


///////////////////////////////////////////////////////////////////////////
//                TRANSMITING CONFIGURATION
///////////////////////////////////////////////////////////////////////////
// size of the transmit buffer. Each transmit buffer contians 2 channel only
// There are two of these (double-buffering)
#define TX_BUF_SIZE     (Layout1_TICK_SIZE * 2)

//////////////////////////////////////////////////////////////////////////
// Transmition buffer declaration
// SPORTs configuration for 4 ADCs
//ADC1 - SPORT2A
//ADC2 - SPORT2B
//ADC3 - SPORT3A
//ADC4 - SPORT3B

static SEG_DRV_DMA_DATA_1 int g_SPORT_TX_BUFFER_1[ANALOG_OUT_STEREO_PAIR_COUNT][TX_BUF_SIZE];
static SEG_DRV_DMA_DATA_2 int g_SPORT_TX_BUFFER_2[ANALOG_OUT_STEREO_PAIR_COUNT][TX_BUF_SIZE];

//---------------------------------------------------------------------------------
//SPORT 1 transmit TCB (Buffer A). 
//left/right transmitter 
static DMA_TCB g_sport2_tx_tcb_A =
{
    0,                                                    // m_CPSPx
    TX_BUF_SIZE,                                        // m_CSPx
    1,                                                     // m_IMSPx
    (unsigned int)(&g_SPORT_TX_BUFFER_1[0][0])            // m_IISPx
};
static DMA_TCB g_sport2_tx_tcb_A_1 =
{
    0,                                                        // m_CPSPx
    TX_BUF_SIZE,                                            // m_CSPx
    1,                                                         // m_IMSPx
    (unsigned int)(&g_SPORT_TX_BUFFER_2[0][0])                // m_IISPx
};

#define SPORT2A_TX_1_START_ADDRESS    (((int)&g_SPORT_TX_BUFFER_1[0][0]) & 0x7FFFF)
#define SPORT2A_TX_1_END_ADDRESS        (((int)&g_SPORT_TX_BUFFER_1[0][TX_BUF_SIZE-1]) & 0x7FFFF)

// SPORT 1 transmit TCB (Buffer B).
// surround left / right transmitter buffer 
static DMA_TCB g_sport2_tx_tcb_B =
{
    0,                                                        // m_CPSPx
    TX_BUF_SIZE,                                            // m_CSPx
    1,                                                         // m_IMSPx
    (unsigned int)(&g_SPORT_TX_BUFFER_1[1][0])                // m_IISPx
};

static DMA_TCB g_sport2_tx_tcb_B_1 =
{
    0,                                                        // m_CPSPx
    TX_BUF_SIZE,                                            // m_CSPx
    1,                                                         // m_IMSPx
    (unsigned int)(&g_SPORT_TX_BUFFER_2[1][0])                // m_IISPx
};

// SPORT 2 transmit TCB (Buffer A).
// center / subwoofer transmitter 
static DMA_TCB g_sport3_tx_tcb_A =
{
    0,                                                        // m_CPSPx
    TX_BUF_SIZE,                                            // m_CSPx
    1,                                                         // m_IMSPx
    (unsigned int)(&g_SPORT_TX_BUFFER_1[2][0])                // m_IISPx
};

static DMA_TCB g_sport3_tx_tcb_A_1 =
{
    0,                                                        // m_CPSPx
    TX_BUF_SIZE,                                            // m_CSPx
    1,                                                         // m_IMSPx
    (unsigned int)(&g_SPORT_TX_BUFFER_2[2][0])                // m_IISPx
};

// SPORT 2 transmit TCB (Buffer B).
// surround back left / right transmitter
static DMA_TCB g_sport3_tx_tcb_B =
{
    0,                                                        // m_CPSPx
    TX_BUF_SIZE,                                            // m_CSPx
    1,                                                         // m_IMSPx
    (unsigned int)(&g_SPORT_TX_BUFFER_1[3][0])                // m_IISPx

};

static DMA_TCB g_sport3_tx_tcb_B_1 =
{
    0,                                                        // m_CPSPx
    TX_BUF_SIZE,                                              // m_CSPx
    1,                                                         // m_IMSPx
    (unsigned int)(&g_SPORT_TX_BUFFER_2[3][0])                // m_IISPx
};

/////////////////////////////////////////////////////////////////////////////////////////

/**
 * Pointer to the Rx Packed data. For in-place packing this points to the DMA
 * buffer that was just in-place repacked.
 */
static int *g_SPORT_Rx;

#ifndef NO_OPTIMIZE
#pragma optimize_for_speed
#endif
static void ServiceAudio(int sig_int)
{
    DoProcessing(g_SPORT_Rx,
                 ((g_iAudioInputMode == AudioIOMode_SPDIF) ?
                  SPDIF_IN_CHANNEL_COUNT :
                  ANALOG_IN_CHANNEL_COUNT));
}        

/**
 * SPORT DMA Interrupt handler.
 *
 * This interrupt handler is called when a DMA transmit and receive operation
 * has completed. This function works out which DMA buffers were used, and calls
 * the user supplied callback procedure with the receive, transmit, and DMA
 * information.
 *
 * Parameter: sig_int            Interrupt number (unused).
 */
#ifndef NO_OPTIMIZE
#pragma optimize_for_speed
#endif
static void AudioReceive_ISR(int sig_int)
{
    
    int *pRx;
    bool bDIR_Locked;
    int spdif_stat, spdif_ctrl;

    if (g_iAudioInputMode == AudioIOMode_SPDIF)
    {
        // read SPDIF RX status
        spdif_stat = RDPORT(DIRSTAT);

#ifdef HANDLE_RECEIVE_ERROR
        // workaround for the SPDIF stream problem
        if (spdif_stat & DIR_NOSTREAM)
        {
            // trying to reset the DIR
            spdif_ctrl = RDPORT(DIRCTL);
            spdif_ctrl |= DIR_RESET;
            WRPORT(DIRCTL) = spdif_ctrl;
            spdif_ctrl &= ~DIR_RESET;
            WRPORT(DIRCTL) = spdif_ctrl;
        }
#endif //#ifdef HANDLE_RECEIVE_ERROR
        bDIR_Locked = ((spdif_stat & DIR_LOCK) != 0);
    }
    else
        bDIR_Locked = true;

    if (bDIR_Locked)
    {
          
        if ( (RDPORT(IISP0A) >= SPORT0A_RX_1_START_ADDRESS)
            && (RDPORT(IISP0A) <= SPORT0A_RX_1_END_ADDRESS))
        {
            pRx = g_SPORT0A_RX_2;
        }
        else
        {
            pRx = g_SPORT0A_RX_1;
        }
    }
    if (bDIR_Locked)
    {
        g_SPORT_Rx = pRx; // Need to pass this statically if we're calling out to a lower interrupt level
        // call lower priority interrupt for rendering
        raise(SIG_USR0);
    }
    else
    {
        SYSAudio_Clear();
    }
}

#ifndef NO_OPTIMIZE
#pragma optimize_for_space
#endif
static void Init_SPORTS(void)
{
    WRPORT(SPMCTL0) = 0 ;
    WRPORT(SPMCTL1) = 0 ;
    WRPORT(SPMCTL2) = 0 ;
    WRPORT(SPMCTL3) = 0 ;

    WRPORT(SPCTL0) = 0 ;
    WRPORT(SPCTL1) = 0 ;
    WRPORT(SPCTL2) = 0 ;
    WRPORT(SPCTL3) = 0 ;
    //============================================================
    //
    // Configure SPORT 0 for input from ADC
    //
    //------------------------------------------------------------
    WRPORT(SPCTL0) = (SCHEN_A | SDEN_A | OPMODE | SLEN32 | SPEN_A | SPORT_DATA_FORMAT);


    // Set circular transfer information for the SPORT0 RX buffers. SPORT0
    // is used for receiving the digital inputs.
    // This is done by having TCB-A point to TCB-B, and TCB-B point to TCB-A
    g_sport0_rx_tcb_A.m_CPSPx = DMA_MakeCPx(&g_sport0_rx_tcb_A_1, 1);
    g_sport0_rx_tcb_A_1.m_CPSPx = DMA_MakeCPx(&g_sport0_rx_tcb_A, 1);
    
    // Enabling Chaining
    // Block A will be filled first
    WRPORT(CPSP0A) = g_sport0_rx_tcb_A.m_CPSPx;

    //============================================================
    //
    // Configure SPORTs 2 & 3 for output to DACs 1-4
    //
    //------------------------------------------------------------
     WRPORT(SPCTL2) = (SPTRAN | OPMODE | SLEN32 | SPEN_A | SCHEN_A | SDEN_A | SPEN_B | SCHEN_B | SDEN_B | SPORT_DATA_FORMAT) ;
    
    //Configuring SPORT1 channel A
    g_sport2_tx_tcb_A.m_CPSPx = DMA_MakeCPx(&g_sport2_tx_tcb_A_1, 1);
    g_sport2_tx_tcb_A_1.m_CPSPx = DMA_MakeCPx(&g_sport2_tx_tcb_A, 1);

    WRPORT(CPSP2A) = g_sport2_tx_tcb_A.m_CPSPx; // write to DAC1

       
    g_sport2_tx_tcb_B.m_CPSPx = DMA_MakeCPx(&g_sport2_tx_tcb_B_1, 1);
    g_sport2_tx_tcb_B_1.m_CPSPx = DMA_MakeCPx(&g_sport2_tx_tcb_B, 1);

    WRPORT(CPSP2B) = g_sport2_tx_tcb_B.m_CPSPx; // write to DAC2
    
    
    //Configuring SPORT2 channel A
    WRPORT(SPCTL3) = (SPTRAN | OPMODE | SLEN32 | SPEN_A | SCHEN_A | SDEN_A | SPEN_B | SCHEN_B | SDEN_B | SPORT_DATA_FORMAT) ;

    g_sport3_tx_tcb_A.m_CPSPx = DMA_MakeCPx(&g_sport3_tx_tcb_A_1, 1);
    g_sport3_tx_tcb_A_1.m_CPSPx = DMA_MakeCPx(&g_sport3_tx_tcb_A, 1);

    // write to DAC3
    WRPORT(CPSP3A) = g_sport3_tx_tcb_A.m_CPSPx;

   
       g_sport3_tx_tcb_B.m_CPSPx = DMA_MakeCPx(&g_sport3_tx_tcb_B_1, 1);
    g_sport3_tx_tcb_B_1.m_CPSPx = DMA_MakeCPx(&g_sport3_tx_tcb_B, 1);

    // write to DAC4
    WRPORT(CPSP3B) = g_sport3_tx_tcb_B.m_CPSPx ;

    //hook up the SPORT0 DMA interrupt
    interruptcb(SIG_SP0, AudioReceive_ISR);

    interruptcb(SIG_USR0, ServiceAudio);    
}

#ifndef NO_OPTIMIZE
#pragma optimize_for_space
#endif
static void StartAudio(void)
{
    WRPORT(CPSP0A) = g_sport0_rx_tcb_A.m_CPSPx;
    WRPORT(CPSP2A) = g_sport2_tx_tcb_A.m_CPSPx;
    WRPORT(CPSP2B) = g_sport2_tx_tcb_B.m_CPSPx;
    WRPORT(CPSP3A) = g_sport3_tx_tcb_A.m_CPSPx;
    WRPORT(CPSP3B) = g_sport3_tx_tcb_B.m_CPSPx;

    sysreg_bit_set(sysreg_LIRPTL, SP0IMSK);  //enable the SP0 interrupt
}

#ifndef NO_OPTIMIZE
#pragma optimize_for_space
#endif
static void StopAudio(void)
{
    WRPORT(CPSP0A) = 0;
    WRPORT(CPSP2A) = 0;
    WRPORT(CPSP2B) = 0;
    WRPORT(CPSP3A) = 0;
    WRPORT(CPSP3B) = 0;
    
    sysreg_bit_clr(sysreg_LIRPTL, SP0I);      //clear any pending interrupt
    sysreg_bit_clr(sysreg_LIRPTL, SP0IMSK);      //disabled the SP0 interrupt
}



///////////////////////////////////////////////////////////////////////
// Exported API
//////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//    SYSAudio_Init
//
//  Description:
//        Initialize the DSP serial ports for audio IO. The audio I/O stream
//    does not flow until SYSAudio_Run() is called.
//
//    Arguments:
//        iAudioIOMode - Auido IO mode set by the framework
//
//    Return:
//        Retrun 1 if failed.
///////////////////////////////////////////////////////////////////////
#ifndef NO_OPTIMIZE
#pragma optimize_for_space
#endif
int SYSAudio_Init(AudioIOMode iAudioIOMode)
{
    g_iAudioInputMode = iAudioIOMode;

    SRU_Init(iAudioIOMode);  //force DAI to SPDIF for making the receiver ready first
   
    SPDIF_Init();   //shall be initialized right after the DAI pin

    //Initialize the SPI port for accessing the Ad1835A CODEC
    Init_AD1835_via_SPI(iAudioIOMode);
    
    Init_SPORTS();
    return 0; 
}

////////////////////////////////////////////////////////////////////////
//    SYSAudio_ChangeMode
//
//  Description:
//        Changes Audio IO mode. The caller of this function should make sure
//        the driver is initialized and stopped (audio is disabled) and the
//        iAudioIOMode is different from current running mode.
//
//    Arguments:
//        iAudioIOMode - New Audio IO mode.
//
//    Return:
//        Return 1 if failed.
///////////////////////////////////////////////////////////////////////
#ifndef NO_OPTIMIZE
#pragma optimize_for_space
#endif
int SYSAudio_ChangeMode(AudioIOMode iAudioIOMode)
{   // Assumes we're initialized and stopped.  
    g_iAudioInputMode = iAudioIOMode;

    // (audio interrupt is disabled)
    // Assumes iAudioIOMode is different from what we're using now
    // Re-route the input ports
    SRU_Init(iAudioIOMode); 

    Init_AD1835_via_SPI(iAudioIOMode);

    return 0;
}

////////////////////////////////////////////////////////////////////////
//    SYSAudio_Run
//
//  Description:
//        Turn on/off the audio. The caller shall keep track of current
//        state of audio driver so as not to halt the audio that has been
//        already disabled.
//
//    Arguments:
//        enable - Turn on the audio if TRUE, turn off the audio otherwise.
//
//    Return:
//        Retrun 1 if failed.
///////////////////////////////////////////////////////////////////////
#ifndef NO_OPTIMIZE
#pragma optimize_for_space
#endif
int SYSAudio_Run(bool enable)
{   
    if (enable)
    {
        StartAudio();
    }
    else
    {
        StopAudio();
    }
    return 0;
}

/////////////////////////////////////////////////////////////////////
// Write mono audio data to an output channel pair.
//
// The left and right audio data pointers can either point to valid audio
// data, or NULL for silence. The number of samples will always be Layout1_TICK_SIZE
//
// The flags word encodes the output channel pair and number of samples to
// write in the following form:
//
//        [31 .. 16]    Output Channel pair.
//        [15 ..  0]  Number of samples.
//
// Parameters:
//    pAudioL                Address of the left audio data.
//    pAudioR                Address of the right audio data.
//    nFlags                Flags word containing channel pair and samples.
/////////////////////////////////////////////////////////////////////////
#ifndef NO_OPTIMIZE
#pragma optimize_for_speed
#endif
void SYSAudio_WriteOutput(
    const float *pAudioL,
    const float *pAudioR,
    unsigned int nFlags)
{
    unsigned int nOutCh;
    int ch;
    int *pTX;
    int *pDst;
    register int i;
    // Get the output channel pair and make sure it is valid for our hardware.
    nOutCh = ((nFlags >> 16) & 0xff);  

    if (nOutCh < (unsigned)ANALOG_OUT_STEREO_PAIR_COUNT) 
    {

        ch = nOutCh;
        if ((RDPORT(IISP2A) >= SPORT2A_TX_1_START_ADDRESS)
            && (RDPORT(IISP2A) <= SPORT2A_TX_1_END_ADDRESS))

        {
            pDst = pTX = (int*)(&g_SPORT_TX_BUFFER_2[ch]);
        }
        else 
        {
            pDst = pTX = (int*)(&g_SPORT_TX_BUFFER_1[ch]);
        }
        
        sysreg_bit_set(sysreg_MODE1, ALUSAT);
        // Check if we have left-channel audio to write.
        if (pAudioL != FW_ADEV_NOCHANGE) 
        {
            if (pAudioL)
            {
                const float *pSrc = pAudioL;
                // Write the audio.
                for (i = 0; i < Layout1_TICK_SIZE * 2; i+=2) 
                {
                    pDst[i] = __builtin_conv_FtoR(*pSrc++) & 0xffffff00;
                }
            }
            else
            {
                for (i=0; i<Layout1_TICK_SIZE*2; i+=2)
                {
                    pDst[i] = 0;
                }
            }
        }
        // Rebased the pointer to point to Right Channel
        pDst = pTX + 1;  

        // Check if we have audio to write.
        if (pAudioR != FW_ADEV_NOCHANGE) 
        {
            if (pAudioR)
            {
                // Write the audio.
                const float *pSrc = pAudioR;
                for (i = 0; i < Layout1_TICK_SIZE * 2; i+=2)
                {
                    pDst[i] = __builtin_conv_FtoR(*pSrc++) & 0xffffff00;
                }
            }
            else
            {
                for (i=0; i<Layout1_TICK_SIZE*2; i+=2)
                    pDst[i] = 0;
            }
        }
        // Clear saturation.
        sysreg_bit_clr(sysreg_MODE1, ALUSAT);
    }
}


/////////////////////////////////////////
// Clear all output.
//
// This function informs the audio device that it should stop outputting
// audio data. 
////////////////////////////////////////////
#ifndef NO_OPTIMIZE
#pragma optimize_for_space
#endif
void SYSAudio_Clear(void)
{
    // Zero the dma buffer itself
    memset(g_SPORT_TX_BUFFER_1,
            0,
            sizeof(g_SPORT_TX_BUFFER_1));
    memset(g_SPORT_TX_BUFFER_2,
            0,
            sizeof(g_SPORT_TX_BUFFER_2));

}

