// Copyright(c) 2005 Analog Devices, Inc. All Rights Reserved.
// This software is proprietary and confidential to Analog Devices, Inc. and its licensors.

// File    : $Id: //depot/development/visualaudio/Platforms/EZKit_21369_Basic/2.5.0/Drivers/Audio/SRU_Init.c#2 $ 
// Part of : VisualAudio V2.5.0 
// Updated : $Date: 2006/10/12 $ by $Author: Fernando $



/**
 *  File name:        SRU_Init.c
 *
 *    Description:    Signal Routine Unit configuration for AVR framework on 21369 EZKit
 *
 *    Owner:            Analog Devices
 *
 *    Revision:
 *
 *    Y. Eric            30 Aug 2005        Initial Revision
 */

#include "AudioIOModes.h"
#include "Driver.h"

#if __VERSIONNUM__ >= 0x07010000  //VDSP++4.0
#include <sru21369.h>
#endif

#if __VERSIONNUM__ >= 0x07000000 && __VERSIONNUM__ < 0x07010000 // VDSP++ 3.5 
#include <sru.h> 
#endif


#ifndef NO_OPTIMIZE
#pragma optimize_for_space
#endif
void SRU_Init(AudioIOMode iAudioInputMode)
{
    //disabled the Pull-up resistors for all 20 pins
    WRPORT(DAI_PIN_PULLUP) = 0x000FFFFF;
    if (iAudioInputMode == AudioIOMode_SPDIF)
    {
        //make sure the pin 18 is outputing
        SRU(LOW, PBEN18_I);
        SRU(LOW, DAI_PB18_I);
        //Connect the on-chip SPDIF receiver to SPDIF jack
        SRU(DAI_PB18_O , DIR_I);
            
        ////////////////////////////////////////////////////////
        //    Configuring the SPDIF input
        ////////////////////////////////////////////////////////
        //receiving SPORT clock
        SRU(DIR_CLK_O ,  SPORT0_CLK_I);
        SRU(DIR_FS_O ,   SPORT0_FS_I ); 
        //input from SPDIF receiver to SPORT0
        SRU(DIR_DAT_O , SPORT0_DA_I );  

        // Clock on pin 7
        SRU(DIR_CLK_O,DAI_PB07_I);

        // Frame sync on pin 8
        SRU(DIR_FS_O,DAI_PB08_I);

        // Tie the pin buffer enable inputs HIGH to drive DAI pins 7 and 8
        SRU(HIGH,PBEN07_I );
        SRU(HIGH,PBEN08_I );

        //------------------------------------------------------------------------
        //  Connect the clock, frame sync, and MCLK from the SPDIF RX directly
        //  to the output pins driving the DACs.
        SRU(DIR_CLK_O,DAI_PB13_I);
        SRU(DIR_FS_O,DAI_PB14_I);
        SRU(DIR_TDMCLK_O,DAI_PB06_I);
        //SRU(DIR_TDMCLK_O,DAI_PB03_I);
        //SRU(DAI_PB02_O,DAI_PB06_I);
        //SRU(HIGH,PBEN03_I);
        //SRU(LOW,PBEN02_I);

        ////////////////////////////////////////////////////////
        //    Configuring the Analog output
        ////////////////////////////////////////////////////////
        //transmitting SPORT clock
        SRU(DIR_CLK_O , SPORT2_CLK_I);//from SPDIF
        SRU(DIR_CLK_O , SPORT3_CLK_I);//from SPDIF
        SRU(DIR_CLK_O , DAI_PB13_I); //clock (BCLK) input from SPDIF on PB03 fed back to DAC via PB13        
        SRU(DIR_FS_O ,  SPORT2_FS_I); //from SPDIF        
        SRU(DIR_FS_O ,  SPORT3_FS_I); //from SPDIF
        SRU(DIR_FS_O ,  DAI_PB14_I); //FS (LRCLK) input from SPDIF on PB04 fed back to DAC via PB14        
        //Analog data output
        SRU(SPORT3_DB_O,DAI_PB09_I);
        SRU(SPORT3_DA_O,DAI_PB10_I);
        SRU(SPORT2_DB_O,DAI_PB11_I);
        SRU(SPORT2_DA_O,DAI_PB12_I);

        ////////////////////////////////////////////////////////
        //    Configuring the SPDIF output
        ////////////////////////////////////////////////////////
        SRU(DIR_CLK_O ,  DIT_CLK_I);
        SRU(DIR_FS_O ,   DIT_FS_I)
        SRU(DIR_TDMCLK_O,DIT_HFCLK_I);   //fedding the MCLK into SPDIF oversampling clock
        
        SRU(SPORT2_DA_O,  DIT_DAT_I);  //By default, wire output #0 and #1 to SPDIF out
        SRU(DIT_O,        DAI_PB01_I);
        SRU(HIGH, PBEN01_I);            //enable SPDIF output
    }
    else
    {

    //  Tie the pin buffer input LOW.
        SRU(LOW,DAI_PB17_I);

    //  Tie the pin buffer enable input LOW
        SRU(LOW,PBEN17_I);

        //  Clock in on pin 7
        SRU(DAI_PB07_O,SPORT0_CLK_I);

        //  Frame sync in on pin 8
        SRU(DAI_PB08_O,SPORT0_FS_I);

        //AD1835 ADC ---> DSP's SPORT0A
        //  Data in on pin 5
        SRU(DAI_PB05_O,SPORT0_DA_I);
        SRU(LOW,DAI_PB05_I);
        SRU(LOW,DAI_PB07_I);
        SRU(LOW,DAI_PB08_I);

        SRU(LOW,PBEN05_I);
        SRU(LOW,PBEN07_I);
        SRU(LOW,PBEN08_I);

        SRU(SPORT3_DB_O,DAI_PB09_I);
        SRU(SPORT3_DA_O,DAI_PB10_I);
        SRU(SPORT2_DB_O,DAI_PB11_I);
        SRU(SPORT2_DA_O,DAI_PB12_I);

        SRU(DAI_PB07_O,DAI_PB13_I);
        SRU(DAI_PB08_O,DAI_PB14_I);
        SRU(DAI_PB17_O,DAI_PB06_I);   //hook xtal to AD1835A MCLK

        SRU(DAI_PB07_O,SPORT2_CLK_I);
        SRU(DAI_PB07_O,SPORT3_CLK_I);
        SRU(DAI_PB08_O,SPORT2_FS_I);
        SRU(DAI_PB08_O,SPORT3_FS_I);

    }
    //------------------------------------------------------------------------
    //  Tie the pin buffer enable inputs HIGH to make DAI pins 9-14 outputs.
    SRU(HIGH, PBEN06_I);
    SRU(HIGH, PBEN09_I); //enable S2Db output on pin 9
    SRU(HIGH, PBEN10_I); //enable S2Da output on pin 10
    SRU(HIGH, PBEN11_I); //enable S1Db output on pin 11
    SRU(HIGH, PBEN12_I); //enable S1DA output on pin 12
    SRU(HIGH, PBEN13_I); //enable BCLK to DACs
    SRU(HIGH, PBEN14_I); //enable LRCLK to DACs


    //Configuring the Digital Peripheral Interface for CODEC, SPDIF devices
    SRU(SPI_MOSI_O,DPI_PB01_I);

    //Connect SPIA's MISO to DPI #2 input. Direction: DPI <---- CODEC
    //MISO in to pin 2
    SRU(DPI_PB02_O, SPI_MISO_I);

    //wired SPIA's clock to the DPI #3. Direction: DPI ----> CODEC
    SRU(SPI_CLK_O, DPI_PB03_I);

    //connect SPI flag pin to CODEC's device select pin
    SRU(SPI_FLG3_O, DPI_PB04_I);

    // Tie pin buffer enable from SPI peipherals to determine whether they are
    // inputs or outputs
    SRU(SPI_MOSI_PBEN_O, DPI_PBEN01_I);
    SRU(SPI_MISO_PBEN_O, DPI_PBEN02_I);
    SRU(SPI_CLK_PBEN_O, DPI_PBEN03_I);
    SRU(SPI_FLG3_PBEN_O, DPI_PBEN04_I);
}
