// Copyright(c) 2005 Analog Devices, Inc. All Rights Reserved.
// This software is proprietary and confidential to Analog Devices, Inc. and its licensors.

// File    : $Id: //depot/development/visualaudio/Platforms/EZKit_21369_Basic/2.5.0/Drivers/SYS_Init.c#2 $ 
// Part of : VisualAudio V2.5.0 
// Updated : $Date: 2006/10/12 $ by $Author: Fernando $




/**
 *  File Name    :    SysLib_Init.c
 *    Module        :    ADSp-21369 EZKit Platform Library
 *    
 *    Description    :    System Level initialization routine
 *
 *    Owner        :    Analog Devices
 *
 *    Revision:
 *
 *    Y. Eric            19 Sep 2005        Updated
 *    Y. Eric            05 Nov 2005        Disabled the SDRAM read Optimization
 *    Y. Eric            07 Nov 2005        Lower the SDRAM clock to 133 MHz and enabled the SDRAM read 
                                    Optimization
 *    Y. Eric            22 Nov 2005        Moved Power Managment Control register setup to assembly file
 */


#include "Driver.h"

#define EXTERNAL_MEMORY_ENABLED 1

#if EXTERNAL_MEMORY_ENABLED
extern setupPLL(unsigned int,unsigned int,unsigned int);

//Initializing the SRAM and SDRAM
#ifndef NO_OPTIMIZE
#pragma optimize_for_space
#endif
static void externalMemoryInit(void)
{

    // Set up PLL for ~331 MHz with SDCLK divisor of 2
    setupPLL(PLLM27,PLLD2,SDCKR2_5);
    WRPORT(PMCTL) |= (CLKOUTEN);  //and start clkout  

    //===================================================================
    //
    // Configure SDRAM Control Register (SDCTL) for the Micron MT48LC4M32
    //
    //  SDCL3  : SDRAM CAS Latency= 3 cycles
    //  DSDCLK1: Disable SDRAM Clock 1
    //  SDPSS  : Start SDRAM Power up Sequence
    //  SDCAW8 : SDRAM Bank Column Address Width= 8 bits
    //  SDRAW12: SDRAM Row Address Width= 12 bits
    //  SDTRAS7: SDRAM tRAS Specification. Active Command delay = 7 cycles
    //  SDTRP3 : SDRAM tRP Specification. Precharge delay = 3 cycles.
    //  SDTWR2 : SDRAM tWR Specification. tWR = 2 cycles.
    //  SDTRCD3: SDRAM tRCD Specification. tRCD = 3 cycles.
    //
    //--------------------------------------------------------------------
    WRPORT(SDCTL) = SDCL3|DSDCLK1|SDPSS|SDCAW8|SDRAW12|SDTRAS7|SDTRP3|SDTWR2|SDTRCD3;  
    
   // Programming SDRAM control registers.
    // RDIV = ((f SDCLK X t REF )/NRA) - (tRAS + tRP )
    // CCLK_SDCLK_RATIO==2
    //ustat4 = 0xA17; // (133*(10^6)*64*(10^-3)/4096) - (7+3) = 2583
#define SDMODIFY 1
    WRPORT(SDRRC) = 0x814 | (SDMODIFY << 17)|SDROPT;  //turn on SDRAM read optimization    

    // Note that MS2 & MS3 pin multiplexed with flag2 & flag3.
    // MSEN bit must be enabled to access SDRAM, but LED8 cannot be driven with sdram
    //settting MSEN bit allows SDRAM access, clearing MSEN bit allow Flag3 to be used
    WRPORT(SYSCTL) |= MSEN;
   
    
        // Mapping Bank 2 to SDRAM, External Bus priority - Core
    // DMA channel Priority - Fixed
    // Make sure that jumper is set appropriately so that MS2 is connected to
    // chip select of 16-bit SDRAM device.
    WRPORT(EPCTL) |= B2SD; //|EPBRDMA|DMAPRCH2;
    WRPORT(EPCTL) &= (~B0SD)|(~B1SD)|(~B3SD);

    //===================================================================
    //
    // Configure AMI Control Register (AMICTL0) Bank 0 for the ISSI IS61LV5128 SRAM
    //
    //  WS2 : Wait States= 2 cycles
    //  HC1  : Bus Hold Cycle (at end of write access)= 1 cycle.
    //  AMIEN: Enable AMI
    //  BW8  : External Data Bus Width= 8 bits.
    //
    //--------------------------------------------------------------------
    //SRAM Settings
    WRPORT(AMICTL0) = WS2|HC1|AMIEN|BW8;

    //===================================================================
    //
    // Configure AMI Control Register (AMICTL) Bank 1 for the AMD AM29LV08
    //
    //  WS23 : Wait States= 23 cycles
    //  AMIEN: Enable AMI
    //  BW8  : External Data Bus Width= 8 bits.
    //
    //--------------------------------------------------------------------
    //Flash Settings
    WRPORT(AMICTL1) = WS23|AMIEN|BW8;
    

}
#endif

#ifndef NO_OPTIMIZE
#pragma optimize_for_space
#endif
void SYS_Init(void)
{
#if EXTERNAL_MEMORY_ENABLED
    externalMemoryInit();
#endif //EXTERNAL_MEMORY_ENABLED
}
 

