// Copyright(c) 2005 Analog Devices, Inc. All Rights Reserved.
// This software is proprietary and confidential to Analog Devices, Inc. and its licensors.

// File    : $Id: //depot/development/visualaudio/Platforms/EZKit_21369_Basic/2.5.0/Drivers/Utilities/SPIDriver.c#2 $ 
// Part of : VisualAudio V2.5.0 
// Updated : $Date: 2006/10/12 $ by $Author: Fernando $

/******************************************************************************************************************
    
    
     Owner:            Analog Devices Inc

    File Group:        Visual Audio Platform Library
    File name:        SPI_Driver.c
      Description:    SPI interfaces implementations

    Revision:

    E. Yin            29 Aug 2005        Initial Revision
******************************************************************************************************************/
#include "Driver.h"   
#include "SPIDriver.h"

/////////////////////////////////////////////////////////////////////////
// SPI_Init
//    SPI congiuration and initializations. It must be called before using
//    SPI for communications
//
// Parameter
//    pSPI_Config_Param - pointer to SPI configuration parameter structure
//
// Return
//    None
/////////////////////////////////////////////////////////////////////////
#ifndef NO_OPTIMIZE
#pragma optimize_for_space
#endif
void SPI_Init(SPIConfigParam *pSPI_Config_Param)
{
    /* First configure the SPI Control registers */
    /* First clear a few registers     */
    WRPORT(SPICTL) = (TXFLSH | RXFLSH) ;  //refresh the transmission and receiving port
    WRPORT(SPIFLG) = 0;
    WRPORT(SPIDMAC) = 0;    //reset the SPI DMA

    /* Setup the baud rate to 1MHz */
    WRPORT(SPIBAUD) = pSPI_Config_Param->unBaudrate;

    
    WRPORT(SPIFLG) = pSPI_Config_Param->unSPIFlag;

    /* Now setup the SPI Control register : 0x5281*/
    //The default setting of CPHASE is 0.
    //if CPHASE set(=1), the slave-select flags manually controlled in software
    //by setting and clearing the appropriate SPIFLGx bit in SPIFLG regiter
    //if CPHASE set(=0), the slave-select flags are enabled by DSxEN bits of SPIFLG
    //are automatically asserted by the SPI port when appropriate, and the SPIFLGx
    //bit is ignored.
   // WRPORT(SPICTL) = (SPIEN | SPIMS | MSBF | WL16 | TIMOD1) ;
     WRPORT(SPICTL) = pSPI_Config_Param->unSPIControl;
}


/////////////////////////////////////////////////////////////////////////
// Write_16Bit_Word
//    Core Transmit a 16-bit word to peripheral device
//
// Parameter
//    p - pointer to buffer to be written
//
// Return
//    None
/////////////////////////////////////////////////////////////////////////
#ifndef NO_OPTIMIZE
#pragma optimize_for_space
#endif
void SPI_Write_16Bit_Word(const unsigned short *p)
{
    WRPORT(TXSPI) = *p;
}

/////////////////////////////////////////////////////////////////////////
// Write_16Bit_Word
//    Core Transmit an array of 16-bit words to peripheral device
//
// Parameter
//    p - pointer to array of data buffer to be written
//  len - length of the array
//
// Return
//    None
/////////////////////////////////////////////////////////////////////////
#ifndef NO_OPTIMIZE
#pragma optimize_for_space
#endif
void SPI_Write_16Bit_Words(const unsigned short *pData, const unsigned int unSize)
{
    unsigned int i;
    for (i = 0; i < unSize; ++i)
    {
        WRPORT(TXSPI) = pData[i] ;
        Delay (100);

        //Wait for the SPI to indicate that it has finished.
        while (!(RDPORT(SPISTAT) & SPIF))
            ;

        Delay (100);
    }
}

/////////////////////////////////////////////////////////////////////////
// SPI_Clear
//    Clear SPI transmit and receiving port
//
// Parameter
//    None
//
// Return
//    None
/////////////////////////////////////////////////////////////////////////
#ifndef NO_OPTIMIZE
#pragma optimize_for_space
#endif
void SPI_Clear(void)
{
    WRPORT(SPICTL) = (TXFLSH | RXFLSH);
}

