// Copyright(c) 2005 Analog Devices, Inc. All Rights Reserved.
// This software is proprietary and confidential to Analog Devices, Inc. and its licensors.

// File    : $Id: //depot/development/visualaudio/common/layoutsupport/2.5.0/Include/Tuning.h#2 $ 
// Part of : VisualAudio Designer v2.0.1
// Updated : $Date: 2006/10/12 $ by $Author: Fernando $

#ifndef _TUNING_H_
#define _TUNING_H_

#include "Tuning_Defines.h"
#include "Layout.h"

typedef struct {
    int nCommandBuffSize;      /* Size of command buffer, not including tuning control */
    unsigned int *pCommandBuff;/* Tuning communication buffer.
                                  * Size must be nCommandBuffSize + 1.
                                  * Final location is used as the tuning control word.
                                  */
    int nLayoutCount;            /* Includes all layouts in run space, whether or
                                  * not loaded
                                  */
    AMF_Layout **pLayouts;       /* Pointer to array of layouts */
    unsigned int iFlags;         /* Flags maintained by AMF_HandleTuningMessage(). */
    volatile unsigned int iSavedToggleBitValue; /* Used to test validity of messages */
    int iInterruptProfileLoopLength; /* Used to profile all of interrupt level processing */
} AMF_Master;

#define AMF_TUNING_CONTROL AMFMaster.pCommandBuff[AMFMaster.nCommandBuffSize]

/* The one and only global AMF object */
extern AMF_Master AMFMaster;

/**
 * Not normally called by user, except when integrating
 * a VisualAudio layout into a custom framework.
 * Function to handle messages, in response to AMFTuningAction_PC_SENDING_COMMAND.
 * The message is assumed to be in AMFMaster.pCommandBuff[].
 * Returns a value suitable for AMF_TUNING_CONTROL.
 */
int AMF_HandleTuningCommand(void);



/**
 * Utilities to disable/enable interrupts.
 */

#ifdef __ADSPBLACKFIN__
/* For blackfin, these are functions, defined in Utilities.asm */
void DISABLE_INTERRUPTS(void);
void ENABLE_INTERRUPTS(void); 
#else
/* For SHARC, these are in-line macros */
#define DISABLE_INTERRUPTS()        asm("bit clr MODE1 IRPTEN; nop; nop;")
#define ENABLE_INTERRUPTS()            asm("bit set MODE1 IRPTEN; nop; nop;")
#endif

/* Same, but allow nesting */
void AMF_BeginAtomic(void);
void AMF_EndAtomic(void);

/**
 * These macros are used internally.
 * Sets toggle bit on the command buffer to synchronize the message
 *
 * Toggling the 0x40000000 bit in the command buffer when replying the message
 */
//This value store the latest state of the toggle bit set by the sender(host).
//The entire command word(32 bit) is passed to this macro
#define AMF_SaveToggleBitValue(var)            (AMFMaster.iSavedToggleBitValue = (var & 0x40000000))
//echoing the toggle bit back to host
#define AMF_RestoreToggleBitValue(var1)        (var1 |= (AMFMaster.iSavedToggleBitValue & 0x40000000))

/*
 * This is used by AMF_ApplyPreset. Returns error code. If success, returns result in pResult.
 * Also increments nAddress to the address of the next 32-bit word.
 * This is used to abstract the data location and access from the applying of the preset.
 * For example, the preset may be located in flash or RAM.
 */
typedef int (*AMF_GetPresetElementFunc)(int *nAddress,int *pResult,void *pContext);

/**
 * Return the cycle counter
 */
__inline static unsigned int AMF_GetCycles(void)
{
  unsigned int _tcount;
#ifndef __ADSPBLACKFIN__
  __asm volatile ("%0=EMUCLK;" : "=d" (_tcount));
#else
  __asm volatile ("%0=CYCLES;" : "=d" (_tcount));
#endif
  return _tcount;
}

/**
 * Load a compiled preset 
 *
 * This function is called in response to a tuning message to load
 * the preset.  If compiled presets from DSP flash or RAM are not supported, 
 * this function should do nothing.  Returns error code or 0 for success.
 * 
 * If high bit of nPresetOffset is set, it will cause interrupts to be masked while 
 * the preset is applied. Else, interrupts will not be masked.
 */
int SYS_ApplyPreset(unsigned int nPresetOffset);

/* This applies a preset or structured setting, as addressed by nAddress.
 * nAddress is passed to pGetFunc, which increments it and returns the value at that
 * address. pContext is passed uninterpreted to getFunc.
 * When used for presets, typically called only via SYS_ApplyPreset().  
 *
 * Returns error code.
 */
int AMF_ApplyPreset(int nAddress,AMF_GetPresetElementFunc getFunc,void *pContext, bool bAtomic);

#endif
