// Copyright(c) 2005 Analog Devices, Inc. All Rights Reserved.
// This software is proprietary and confidential to Analog Devices, Inc. and its licensors.

// File    : $Id: //depot/development/visualaudio/common/layoutsupport/2.5.0/Source/Utilities.asm#2 $ 
// Part of : VisualAudio V2.5.0 
// Updated : $Date: 2006/10/12 $ by $Author: Fernando $

#ifndef __ADSPBLACKFIN__
#include <asm_sprt.h>
#include "processor.h"

.section /dm mod_fastsdata ; 

.global _g_nInterruptsAlreadyDisabled;
.global _g_nAtomicNestDepth;

.var _g_nInterruptsAlreadyDisabled = 0x0;  //initiailize to 0
.var _g_nAtomicNestDepth = 0x0;  //initiailize to 0

// This logic is based on the following assumptions:
// - BeginAtomic can only be called without an intervening EndAtomic if it 
//   is done by the same "thread" that called the first one, because otherwise
//   the fact that the interrupts were disabled prohibits any other "threads"
//   from executing.  In other words, the usage will at worst be "nested" within
//   the same thread.
// - The code assumes that other code ahead of the first call to BeginAtomic may 
//   already have disabled interrupts, so it checks for the interrupts already
//   being disabled, and remembers that fact, not reenabling interrupts upon
//   reaching the last EndAtomic if so.

// _AMF_BeginAtomic:
//         test and clear IRPTEM
//         if depth == 0 then alreadyDisabled = !(IRPTEN test)
//         depth++

.section/pm mod_fastcode;
.global _AMF_BeginAtomic;
_AMF_BeginAtomic:
leaf_entry;
     bit tst MODE1 IRPTEN;
    bit clr MODE1 IRPTEN; // the rest of the fn done with interrupts disabled
    r0 = 1;
    i4 = _g_nInterruptsAlreadyDisabled;
    if tf r0=r0-r0;            // tf has the result of the bit tst operation
    r1 = dm(_g_nAtomicNestDepth);
    r1 = pass r1;
    if eq dm(i4,m5) = r0;   //set the "was already disabled" flag
    r1 = r1+1;
    dm(_g_nAtomicNestDepth) = r1;
leaf_exit;
_AMF_BeginAtomic.END:

// _AMF_EndAtomic;
//         if depth > 0 
//            depth--
//            if (depth > 0 || alreadyDisabled) 
//                return
//             set IRPTEN
//

.global _AMF_EndAtomic;
_AMF_EndAtomic:
leaf_entry;
    r0 = dm(_g_nAtomicNestDepth);
    r0 = pass r0;
    if le jump done;
    r0 = r0-1;
    dm(_g_nAtomicNestDepth)=r0;
    if gt jump done;
    r1 = dm(_g_nInterruptsAlreadyDisabled);
    r1 = pass r1;
    if ne jump done;
    bit set MODE1 IRPTEN; // everything up to here done while interrupts still disabled
done:
leaf_exit;
_AMF_EndAtomic.END:

#else
#include <defBF533.h>
        
//BLACKFIN
.section data1;
.align 4;
.global _g_nInterruptsAlreadyDisabled;
.global _g_nAtomicNestDepth;
.global _ImaskSave;
.global _ImaskSave2;

.byte4 _g_nInterruptsAlreadyDisabled = 0x0;
.byte4 _g_nAtomicNestDepth = 0x0;  //initiailize to 0

.byte4 _ImaskSave = 0;
.byte4 _ImaskSave2 = 0;

.section program;

.align 4;
.global _AMF_BeginAtomic;
_AMF_BeginAtomic:
    
    p0.h = (IMASK >> 16) & 0xFFFF;
    p0.l = IMASK & 0xFFFF;
    r0 = [p0];
    r1 = 0x001F;
    cc = r0 == r1;
    
    SSYNC;
    cli r0;
    
    p1.l = _g_nAtomicNestDepth;
    p1.h = _g_nAtomicNestDepth;
    r1 = [p1];        //Depth

    p0.l = _g_nInterruptsAlreadyDisabled;
    p0.h = _g_nInterruptsAlreadyDisabled;
    if cc jump set_flag;    
        r2 = 0;
        jump no_set_flag;
    
set_flag:
    r2 = 1;

no_set_flag:    
    cc = r1 == 0;
    if !cc jump skip;
        [p0] = r2;
        
        p0.l = _ImaskSave;
        p0.h = _ImaskSave;
        [p0] = r0;

skip:
    r1 += 1;    //Depth ++
    [p1] = r1;

END_BeginAtomic:
    rts;
    nop;
    
_AMF_BeginAtomic.end:

    
.global _AMF_EndAtomic;
_AMF_EndAtomic:
    p1.l = _g_nAtomicNestDepth;
    p1.h = _g_nAtomicNestDepth;
    r0 = [p1];
    
    cc = r0 <= 0;
    if cc jump END_Atomic;  
    
    r0 += -1;
    [p1] = r0;    //Depth --
    
    cc = r0 <= 0;
    if !cc jump END_Atomic;
    
    p0.l = _g_nInterruptsAlreadyDisabled;
    p0.h = _g_nInterruptsAlreadyDisabled;
    r0 = [p0];
    cc = r0 == 0;
    if !cc jump END_Atomic;
    
    p0.l = _ImaskSave;    //Get the right IMASK from the ImaskSave stack with the right depth
    p0.h = _ImaskSave;
    r0 = [p0];

    SSYNC;
    sti r0;
    
END_Atomic:    
    rts;
    nop;
    
_AMF_EndAtomic.end:
    

.global _DISABLE_INTERRUPTS;
_DISABLE_INTERRUPTS:
    SSYNC;
    
    cli r0;
    
    p0.l = _ImaskSave2;
    p0.h = _ImaskSave2;
    [p0] = r0;

    rts;    
    nop;
    
_DISABLE_INTERRUPTS.end:

    
.global _ENABLE_INTERRUPTS;
_ENABLE_INTERRUPTS:

    SSYNC;
    
    p0.l = _ImaskSave2;
    p0.h = _ImaskSave2;
    r0 = [p0];

    sti r0;
    
    rts;
    nop;
    
_ENABLE_INTERRUPTS.end:

    
.global _calcCRC32;
_calcCRC32:

    L0 = 0;
    p1 = r1;    //length
    I0 = r0;    //ptr to data
    
    r0 = 0;
    nop; nop; // Explicit stall to suppress "Informational ea1056" message.

    lsetup(A_32, EndA_32) lc1 = p1;
        A_32:        r2 = [I0++]; 
    EndA_32:        r0 = r2 + r0(NS);
    
    r0 = ~ r0;
    r0 += 1;    
    rts;
    nop;
    
_calcCRC32.end:

#endif
