// Copyright(c) 2005 Analog Devices, Inc. All Rights Reserved.
// This software is proprietary and confidential to Analog Devices, Inc. and its licensors.

// File    : $Id: //depot/development/visualaudio/Platforms/EZKit_21369_Basic/2.5.0/main.c#2 $ 
// Part of : VisualAudio V2.5.0 
// Updated : $Date: 2006/10/12 $ by $Author: Fernando $


/*************************************************************************************
Project name:       VisualAudio Layout example

Hardware:           EZKit 21369

Date :              9/01/2005

Description :   This example program sets up the Audio device driver, which configures the codecs
                via SPI ports and SPORT ports. Audio data is transferred through DMA.
                This program passes the input audio data to the VisualAudio layout 
                processing function, and sends output data through DMA. 
                
*********************************************************************************/

/*********************************************************************

Include files

*********************************************************************/

#include "processor.h"
#include <signal.h>
#include <string.h>

#include "Basic.h"
#include "VALayouts.h"              /* In VAGenerated/ directory */
#include "Tuning.h"                 /* In LayoutSupport/Include directory */

#define TUNING_ENABLED 1        // Tuning via background telemetry channel
#define POWER_SAVING_ENABLED 0  // Power saving interferes with statistical profiler so off by default.

/* Tuning command buffer.  Size is specified in VALayouts.h. */
#if TUNING_ENABLED
static SEG_MOD_SLOW_SDATA unsigned int tuningCommandBuffer[(AMF_MAX_MSG_SIZE+1)];
#else
#define NULL 0
#endif

/* Array of layouts. */
AMF_Layout *layouts[] = AMF_LAYOUT_LIST_INITIALIZER;

/**
 * The one and only master object for the VA AudioProcessing (AMF) library.
 */
AMF_Master AMFMaster =
{
    AMF_MAX_MSG_SIZE,
#if TUNING_ENABLED
    tuningCommandBuffer,
#else
    NULL,
#endif    
    1,
    layouts,
    0
};

#if TUNING_ENABLED
/* BTC Definitions */
#include "btc.h"
BTC_MAP_BEGIN
//             Channel Name, Starting Address,               Length
BTC_MAP_ENTRY("VisualAudio", (long)tuningCommandBuffer, sizeof(tuningCommandBuffer)) 
// Insert any addition BTC_MAP_ENTRY's here
BTC_MAP_END
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

#if Layout1_SCRATCH_COUNT
static SEG_MOD_FAST_SDATA AMF_Signal fastDMMemArray[Layout1_SCRATCH_COUNT];
#define fastDMMem &fastDMMemArray[0]
#else
#define fastDMMem NULL
#endif

#if Layout1_PM_SCRATCH_COUNT
static SEG_MOD_FAST_SDATA_PM AMF_Signal PMMemArray[Layout1_PM_SCRATCH_COUNT];
#define PMMem &PMMemArray[0]
#else
#define PMMem NULL
#endif

#if Layout1_SLOW_SCRATCH_COUNT
static SEG_MOD_SLOW_SDATA AMF_Signal slowDMMemArray[Layout1_SLOW_SCRATCH_COUNT];
#define slowDMMem &slowDMArray[0]
#else
#define slowDMMem NULL
#endif

static SEG_MOD_FAST_SDATA 
    AMF_Signal processingBuffer[MAX(Layout1_INPUT_PIN_COUNT,Layout1_OUTPUT_PIN_COUNT)][Layout1_TICK_SIZE];
/*********************************************************************
*
*    Function:    main
*
*********************************************************************/
#ifndef NO_OPTIMIZE
#pragma optimize_for_space
#endif
void main(void)
{
    SYS_Init();
    extern float debugbuffer[50];
    // init processing
    Layout1.pProcessingBuffer = (AMF_Signal *)processingBuffer;
#if TUNING_ENABLED
    btc_init();
    /* Hook the BTC interrupt */
    FW_Interrupt(SIG_EMUL,   btc_isr);
#endif
    AMF_Layout_Init(&Layout1,fastDMMem,PMMem,slowDMMem);

    // initialize profiling
    AMF_Layout_InitProfile(&Layout1,Layout1_TICK_SIZE);

    // initialize the audio driver with analog as input mode 
    SYSAudio_Init(DEFAULT_AUDIO_IO_MODE);

    for (;;)
    {
        // infinite loop while audio flows in background. Poll for tuning commands.
#if TUNING_ENABLED
        if (AMF_TUNING_CONTROL == AMFTuningAction_PC_SENDING_COMMAND)
            AMF_TUNING_CONTROL = AMF_HandleTuningCommand();
#endif
        DoControlCode(); // See DoControlCode.c
#if POWER_SAVING_ENABLED
        asm("idle;"); // Save power... wake up only once per DMA
        // Note that due to a chip anomaly, doing this interferes with the 
        //      VisualDSP++ statistical profiler
#endif
    }    
}

#define    ADEV_WRITE_FLAGS(ch, samps)        ((ch) << 16 | (samps))

#ifndef NO_OPTIMIZE
#pragma optimize_for_speed
#endif
void DoProcessing(int *pInputBuffer,int nInputChannels)
{
    // Called from ISR
    int i, j, flags, chan;
    int lim;
    unsigned int startCycles;

    if ((AMFMaster.iFlags & AMFFlag_PAUSED) != 0) {    // Can be paused by (e.g.) demand render mode
        SYSAudio_Clear();
        return;
    }
    
    startCycles = AMF_Layout_BeginProfile(&Layout1);

    // Copy from interleaved DMA buffer to non-interleaved processing buffer and convert to float
    lim = MIN(Layout1_INPUT_PIN_COUNT,nInputChannels);
    for (chan=0; chan<lim; chan++)
        for (j=0; j<Layout1_TICK_SIZE; j++) 
            Layout1.pProcessingBuffer[chan*Layout1_TICK_SIZE+j] = AMF_Fract32ToSignal(pInputBuffer[chan+j*nInputChannels]);
    if (nInputChannels < Layout1_INPUT_PIN_COUNT)
        memset(&Layout1.pProcessingBuffer[lim],0,Layout1_INPUT_PIN_COUNT-nInputChannels);

    // Do renderng
    AMF_Layout_Render(&Layout1, Layout1_TICK_SIZE);

    // Copy to output
    chan = 0;
    for (i = 0; i < Layout1_OUTPUT_PIN_COUNT; i+=2) 
    { 
        flags = ADEV_WRITE_FLAGS(chan, Layout1_TICK_SIZE); 
        /* Write both channels */ 
        SYSAudio_WriteOutput(processingBuffer[i], processingBuffer[i+1], flags); 
        chan++; 
    } 
#if (Layout1_OUTPUT_PIN_COUNT & 1) // Odd number of channels? 
    flags = ADEV_WRITE_FLAGS(chan, Layout1_TICK_SIZE); 
    /* Write the last (left) channel only. */ 
    SYSAudio_WriteOutput(processingBuffer[i], FW_ADEV_NOCHANGE, flags); 
#endif

    AMF_Layout_EndProfile(&Layout1,startCycles);
}
