// Copyright(c) 2005 Analog Devices, Inc. All Rights Reserved.
// This software is proprietary and confidential to Analog Devices, Inc. and its licensors.

// File    : $Id: //depot/development/visualaudio/Platforms/EZKit_21369_Basic/2.5.0/preset.c#2 $ 
// Part of : VisualAudio V2.5.0 
// Updated : $Date: 2006/10/12 $ by $Author: Fernando $


/*
 *  Name:            Preset.c
 *    Description:    Load the preset from the flash on 21369 EZKIT
 *
 *    Owner:            Analog Devices
 *
 *    Revision:
 *
 *    E. Yin            23 Sep 2005            Initial Revision
 */


#include <builtins.h>
#include <stdlib.h>
#include "Basic.h" // Has definition for PRESETS_IN_FLASH
#include "Driver.h"
#include "Tuning.h"
#include "VALayout1Presets.h"

#if (Layout1_COMPILED_PRESET_COUNT==0)

// Dummy function
int SYS_ApplyPreset(unsigned int nPresetOffset) {
    return 0;
}

#else

// Flash is 32-bit word addressable
#define NEXT_ADDRESS(_addr) (_addr) += 1

#define PRESET_BASE_ADDRESS ((unsigned int)(&Layout1_Presets[0]))

// #define PRESET_FLASH_ADDRESS 0x04000000 // This is starting address of the exteranl flash on the 369 EZKit 

#ifndef NO_OPTIMIZE
#pragma optimize_for_space
#endif
static int getPresetElement(int *nAddress,int *pResult,void *pContext)
{
    int n;
    int addr = *nAddress;
    n = *(int*)addr; // Read word 
    NEXT_ADDRESS(addr);
    *nAddress = addr;
    *pResult = n;
    return E_NO_ERROR;
}


////////////////////////////////////////////////////////////////////////////////////////////
//  SYS_ApplyPreset
//
//    Description:
//        Read preset from flash or ram and apply them to the specified destination
//
//Arguments:    
//    nPresetOffset - Offset value into the preset buffer, in words (not bytes). This is a linear address.
//
// Returns:
//    Error code.
////////////////////////////////////////////////////////////////////////////////////////////

#ifndef NO_OPTIMIZE
#pragma optimize_for_space
#endif
int SYS_ApplyPreset(unsigned int nPresetOffset)
{
    // get the flash or ram address
    // Mask the high bit of nPresetOffset to get actual offset
    int addr = 0;
    
    // If the high bit of nPresetOffset is set, it will cause interrupts to be masked while the preset is applied.  
    // Otherwise, interrupts will not be masked.
    bool bAtomic = ((nPresetOffset & (unsigned int)AMFStructuredSetting_ATOMIC) != 0);
    
    // get the flash or ram address
    // Mask the high bit of nPresetOffset to get actual offset
    addr = (nPresetOffset & ~AMFStructuredSetting_ATOMIC) + PRESET_BASE_ADDRESS;
    return AMF_ApplyPreset(addr, (AMF_GetPresetElementFunc)getPresetElement, NULL, bAtomic);
}
#endif
