%Model of nonlinear loudspeaker
%Using linear parameters from
%Using nonlinear parameters from
%X(n) - state vector
%Bl - force factor vector
%c - total compliance vector
%b - total Bl vector
%M - total mass
%Re - Electrical resistance
%Rt - Mechanical resistance
%R2 - electrical resistance due the eddy current losses
%fs - sampling rate

%Selecting Loudspeaker type
speaker = 8480277;      

%sampling frequency
%   -for inputsignal w(n), loudspeakermodel and nonlinear compensator
fs = 48000;             

%Table's for nonlinear parameters - properties.
table_size = 32000;     % tabel-size for nonlinear parameters
max_excursion = 0.02;   % [m]
stepsize_table = (2*max_excursion)/table_size;

%Loading Linear Parameters (Bl_lin,C_lin,Le_lin is not used later)   
[Bl_lin,C_lin,Le_lin,M,Re,Rt,L2_0,R2_0] = Get_Linear_Parameters(speaker); 

%Loading Nonlinear Parameters (Polynomial coefficients and tabels)
[a,b,c,K,Le,Bl,C,dLe] = Get_Nonlinear_Parameters(speaker,8,8,8,table_size,max_excursion,stepsize_table);%Le,Bl,C
C_str = 1;     %stretching of compliance
C_sca = 1;     %scaling of compliance
% [C] = compliance_tuning(C,stepsize_table, table_size, C_str, C_sca);
%Plot_Nonlinear_Parameters(a,b,c,k,fs)

%Input signal, frequencies(Hz) and Input voltages, amplitude(V)(convert from rms to amplitude)  
f = [53*1];% 25 30 35 40 45 50 55 60 65 70 75 80 90 100 150 200 300 400 500 700 900 1000 1200 1400 1600 1800 2000 3000];    
fm = [1000];         
A = 4;  Am =4;  %
n = 1:96000;
result = zeros(6,length(fm));     
k=1;
for j=1:length(fm)
    %Generates input signal (driver voltage - u_D)
    w = A*sin(2*pi*f(k)*n/fs) + Am*sin(2*pi*fm(j)*n/fs);
    
    %Loadspeaker Model
    [X] = Nonlinear_Loudspeaker_Model(w,L2_0,R2_0,M,Re,Rt,fs,Le,Bl,C,dLe,table_size,max_excursion,stepsize_table,C_str);
    %[X] = Nonlinear_Loudspeaker_Model_Simplified(w,L2_0,R2_0,M,Re,Rt,fs,Le,Bl,C,dLe,table_size,max_excursion,stepsize_table);

    %Nonlinear Compensator
    [X_comp,u] = Nonlinear_Compensator(w,Le,Bl,C,dLe,L2_0,R2_0,M,Re,Rt,fs,table_size,max_excursion,stepsize_table);
    %[X_comp,u] = Nonlinear_Compensator_Simplified(w,Le,Bl,C,dLe,L2_0,R2_0,M,Re,Rt,fs,table_size,max_excursion,stepsize_table);
    %[X_comp,u] = Nonlinear_Compensator_Simplified_Polynom(w,a,b,c,Le,Bl,C,dLe,L2_0,R2_0,M,Re,Rt,fs,table_size,max_excursion,stepsize_table);
    
    %Processed signal on linear loudspeaker model
    %[X_processed] = Linear_Loudspeaker_Model(u,a,b,c,M,Re,Rt,R2_0,L2_0,fs);
    
    %Plot timedomain
    figure(4);
        plot([1.02*u(1:17001)' w(1:17001)'])% '
        axis([4000 10000 -18.5 18.5]);   grid;
        title('blue-compensated driver voltage, u_D(n), green-uncompensated driver voltage w(n)')
        ylabel('Voltage[V]')
        xlabel('Discrete time')
    
%     figure(5);
%         plot([100*X(3,1:17001)' X(1,1:17001)'])% 1.02*u(1:17001)'w(1:17001)'
%         axis([4000 10000 -0.35 0.35]);   grid;
%         title('blue - membrane position, u_D(n), green - current)')
%         ylabel('Voltage[V]')
%         xlabel('Discrete time')
        
%Frequency analyse
    Nf = 2^17;
    nx = 1:(Nf/2);
    %Output, loudspeaker model
    y = fft(X(3,7001:64000).*hann(57000)',Nf);
    Pyy = (y.*conj(y))/Nf;
    Prad = (((Pyy(1:Nf/2))).*((nx*fs/Nf*2*pi).^4)); %*1/2*1.2*(0.0525^2*pi)^2/(2*pi*344)
    %Compensated Output, loudspeaker model + compensator
    nx_comp = 1:15000/2;
    y_comp = fft(X_comp(3,7001:64000).*hann(57000)',Nf);
    Pyy_comp = (y_comp.*conj(y_comp))/Nf;
    Prad_comp = (((Pyy_comp(1:Nf/2))).*((nx*fs/Nf*2*pi).^4)); %*1/2*1.2*(0.0525^2*pi)^2/(2*pi*344)
    %Output, processed signal on linear loudspeaker model
%     y_processed = fft(X_processed(3,7001:64000).*hann(57000)',Nf);
%     Pyy_processed = (y_processed.*conj(y_processed))/Nf;
%     Prad_processed = (((Pyy_processed(1:Nf/2))).*((nx*fs/Nf*2*pi).^4));

    %Frequency  plot
    figure(1);
        plot(nx_comp*fs/Nf,10*log10(Prad(1:15000/2)));
        axis([0 2500 -90 100]);    grid minor; 
        title('The simulated power radiation - without compensation')
        ylabel('Radiated power [dB]')
        xlabel('Frequency[Hz]')
    figure(22);
        plot(nx_comp*fs/Nf,10*log10(Prad_comp(1:15000/2)));
        axis([0 3000 -90 100]);    grid minor;
        title('The simulated power radiation - with compensation')
        ylabel('Radiated power [dB]')
        xlabel('Frequency [Hz]')
%      figure(3);
%         plot(nx_comp*fs/Nf,10*log10(Prad_processed(1:15000/2)));
%         axis([0 3000 -90 100]);    grid minor; 
%         title('The simulated power radiation - Prosecced signal on linear loudspeaker')
%         ylabel('Radiated power [dB]')
%         xlabel('Frequency[Hz]')

    %THD-Calculation (Harmonic Distortion)
    nh = 5;                     % number of harmonics in distortion calculation.
    nf = 1;                     % number of neighbour frequencies in distortion.
    dist = zeros(1,nh+1);       % Amplitude for fundamental, first harmonic, second harm...
%     [z, dist(:)] = THD((Prad(1:Nf/2).^0.5),f(k),fs,Nf/2,nh,nf);
%     display([' The harmonic distortion in speaker is ',num2str(z,'% 2.3g'),'% @ ',num2str(A/sqrt(1),'%2.2g'),' Volts, @ ', num2str(f),' Hz']) %when calculated to the ',num2str(nh,'%2.2g'),'th harmonic
%     result(5,k) = z;  
%     [z, dist(:)] = THD((Prad_comp(1:Nf/2).^0.5),f(k),fs,Nf/2,nh,nf);
%     display([' The harmonic distortion in compensated speaker is ',num2str(z,'% 2.3g'),'% @ ',num2str(A/sqrt(1),'%2.2g'),' Volts, @ ', num2str(f),' Hz']) %when calculated to the ',num2str(nh,'%2.2g'),'th harmonic
%     result(6,k) = z;      

    %IMD-Calculation (Intermodulation Distortion)
    z = IMD((Prad(1:Nf/2).^0.5),f(k),fm(j),fs,Nf/2,nh,nf);
    display([' The intermodulation in speaker is ',num2str(z,'% 2.3g'),'% @ ',num2str(A/sqrt(1),'%2.2g'),' Volts, @ ', num2str(fm(k)),' Hz']) %when calculated to the ',num2str(nh,'%2.2g'),'th harmonic
    result(5,j) = z;    
    z = IMD((Prad_comp(1:Nf/2).^0.5),f(k),fm(j),fs,Nf/2,nh,nf);
    display([' The intermodulation distortion in compensated speaker is ',num2str(z,'% 2.3g'),'% @ ',num2str(A/sqrt(1),'%2.2g'),' Volts, @ ', num2str(fm(k)),' Hz']) %when calculated to the ',num2str(nh,'%2.2g'),'th harmonic
    result(6,j) = z; 
    
    
     result(1,j) = fm(j);
    result(2,j) = f(k);
    result(3,j) = max(X(3,17001:32000))*1000;
    result(4,j) = min(X(3,17001:32000))*1000;
    result
end
