function [OUT,u] = Nonlinear_Compensator_Simplified_Polynom(w,a,b,c,Le_tab,Bl_tab,C_tab, dLe_tab,L2_0,R2_0,M,Re,Rm,fs,table_size,max_x,stepsize_table);
    %Linear Feedback compensator (w,a,b,c,Le,Bl,C,L2_0,R2_0,M,Re,Rt,fs);
    %   u - compensated output voltage
    %   w - input voltage
    %   Bl_tab - force factor vector, tabel in mm, +-10mm
    %   Le_tab - inductance vector, tabel in mm, +-10mm
    %   C_tab - compliense vector, tabel in mm, +-10mm
    %   a - inductance polynomes, vector
    %   c - compliance polynomes, vector
    %   b - force factor polynomes, vector
    %   M - total mass
    %   Re - Electrical resistance
    %   Rm - Mechanical resistance
    %   R2 - electrical resistance due the eddy current losses
    %   fs - sampling rate
     
     Ts = 1/fs;                      % Stepsize
    Le_0 = a(1);      % inductance, converting to Henry
    Bl_0 = b(1);            % force factor 
    C_0 = c(1);       % compliance, converting to m/N
    K_0 = 1/C_0;      % stiffness N/m
        
    fase = 0;
    v = zeros(1,length(w));  %   LD(linear dynamics) output vector
    u = zeros(1,length(w));  %   compensator output vector 
    X = zeros(3,length(w));  %   X-vector      
    
    for k=1:fase+1
    
        X(:,k)= [  0           %   x1 - current   
                   0           %   x3 - excution
                   0  ];       %   x4 - velosity  
    end

    for k=1:64000
        %Loading nonlinear parameters from tables      
        Bl = polyval(fliplr(b),X(2,k));
        Le = (polyval(fliplr(a),X(2,k)));    
        C = polyval(fliplr(c),X(2,k));
        K = 1/C;
        K_next = 1/polyval(fliplr(c),X(2,k)+ 0.000001);
        dLe = polyval(fliplr(a(2:9).*[1 2 3 4 5 6 7 8]),X(2,k));   
        dBl = polyval(fliplr(b(2:9).*[1 2 3 4 5 6 7 8]),X(2,k));
        dC = polyval(fliplr(c(2:9).*[1 2 3 4 5 6 7 8]),X(2,k));
        dK = (K_next-K)/0.000001;
        ddLe = polyval(fliplr(a(3:9).*[2 6 12 20 30 42 56]),X(2,k));
               
        %constants for speaker model
        L2 = Le*L2_0/Le_0;       
        R2 = Le*R2_0/Le_0;
        dL2 = dLe*L2_0/Le_0; 
        
        %compensator
        u(k) = Le/(Bl+dLe*X(1,k))*(X(3,k)*(X(2,k)*dK + (K-K_0) - dBl*X(1,k) ...
              - ddLe*X(1,k)^2/2 - Bl_0^2/Le_0) + Re*X(2,k)*(K-K_0)/Le_0 - ...
              Re*X(1,k)*(2*Bl+dLe*X(1,k))/(2*Le_0) + Bl_0*w(k)/Le_0) + Re*X(1,k) + ...
              Bl*X(3,k) + dLe*X(1,k)*X(3,k);% + R2*(X(1,k)-X(2,k));   
%                    u(k) =w(k);             
        
        %calculation of F and G matrix values
        F=[(1-((Re/Le)*Ts)) 0 -Ts/Le*(X(1,k)*dLe+Bl); 
            0 1 Ts;  
            (Ts*Bl/M) (-Ts/(C*M)) (1-Rm*Ts/M)];
            G=[Ts/Le ; 0 ; 0];
        %calculating the state variables
        X(:,k+1) = F*X(:,k) + G*u(k);
    
        
    end
    OUT(1,:) = X(1,:);
    OUT(3,:) = X(2,:);
    OUT(4,:) = X(3,:);
end